/* 
  xjuggle.cc
  .
  This program demonstrates juggling patterns
  (with a nice X user interface)
  .
  This file contains the X user interface code and the
  main routine.
  .
 */


/* 
  Xlib include files 
  */
extern "C" {
#include<X11/Xlib.h>
};

/* 
  Xt include files 
  */
extern "C" {
#include<X11/Intrinsic.h>
#include<X11/StringDefs.h>
};

/* 
  Athena Widgets 
  */
extern "C" {
#include<X11/Xaw/Form.h>
#include<X11/Xaw/Label.h>
#include<X11/Xaw/Scrollbar.h>
};


/* 
  Custom Widgets 
  */
extern "C" {
#include "widgets/Juggle.h"
};


/* 
  Standard C include files 
  */
extern "C" {
#include<stdio.h>
#include<stdlib.h>
#include<math.h>
};



/*
   C++ include files
   */

/*
   functions 
   */

void scrollbar1_jumpProc(Widget w, 
			 Widget j, /* juggle_widget */
			 float* percent_ptr /* float */
			 )
{
  /* WARNING: hardcoded value and string reather than #defined value */
  float f;
  f = exp(8 * ( *percent_ptr) * log(4));
  XtVaSetValues(j,"gf_ptr", (XtPointer) &f, NULL);
};

void scrollbar2_jumpProc(Widget w, 
			 Widget j, /* juggle_widget */
			 float* percent_ptr /* float */
			 )
{
  /* WARNING: hardcoded value and string reather than #defined value */
  float f;
  f = - exp((8 * ( *percent_ptr) + 2) * log(2));
  XtVaSetValues(j,
		"tv_ptr",  &f,
		NULL);
};


void quit_application(Widget w, XEvent e, char* params, Cardinal* num_params){
  exit(0);
};

int main(int argc, char** argv){
  
  int i, j;
  XtAppContext app_context;
  XrmOptionDescRec* options = NULL;
  char** fallback_resources = NULL;
  ArgList args;
  Widget top_widget;

  /* 
     X setup 
    */

  top_widget = XtAppInitialize(&app_context, "XJuggle", options, 0, \
			       &argc, argv, fallback_resources, args, 0);
  static XtActionsRec Quit_Application[] = {
    {"quit", quit_application}
  };
  XtAppAddActions(app_context, Quit_Application, XtNumber(Quit_Application));
  
  Widget form_widget;
  form_widget = \
    XtVaCreateManagedWidget( "form", formWidgetClass, top_widget, NULL);
  
  Widget label_widget;
  label_widget = \
    XtVaCreateManagedWidget( "label", labelWidgetClass, form_widget, 
			    XtNlabel, "XJuggle 0.7",
			    NULL);
  
  Widget juggle_widget;
  juggle_widget = \
    XtVaCreateManagedWidget( "juggle", juggleWidgetClass, form_widget, NULL );
  
  /* scrollbars to control various parameters */
  Widget scroll_label1, scroll_label2;
  Widget scrollbar1, scrollbar2;
  label_widget = \
    XtVaCreateManagedWidget( "scroll_label1", labelWidgetClass, \
			    form_widget, NULL);
  scrollbar1 = \
    XtVaCreateManagedWidget( "scrollbar1", scrollbarWidgetClass, 
			    form_widget, XtNorientation, XtorientHorizontal,
			    NULL);
  XtAddCallback(scrollbar1, XtNjumpProc, \
		scrollbar1_jumpProc, juggle_widget);

  label_widget = \
    XtVaCreateManagedWidget( "scroll_label2", labelWidgetClass, \
			    form_widget, NULL);
  scrollbar2 = \
    XtVaCreateManagedWidget( "scrollbar2", scrollbarWidgetClass, 
			    form_widget, XtNorientation, XtorientHorizontal,
			      NULL);
  XtAddCallback(scrollbar2, XtNjumpProc, \
		     scrollbar2_jumpProc , juggle_widget);
  

  XtRealizeWidget(top_widget);

  XtAppMainLoop(app_context);


}


